<nav>
    <hstack spacing=s align-x=center>
        <a href="/">BrandX</a>
        <spacer></spacer>
        <a href="{% url 'bookmarks:bookmarked_objs' %}">Saved</a>
        <a href="{% url 'bookmarks:annotated_tags' %}">Tags</a>
        {% load account %}
        {% if user.is_authenticated %}
            <a href="{% url 'account_logout' %}">Logout</a>
        {% else %}
            <a href="{% url 'account_login' %}">Login</a>
            <a href="{% url 'account_signup' %}">Signup</a>
        {% endif %}
    </hstack>
</nav>
